/*

  Programma Preda_predatore.ccp:
  integra le equazioni di un sistema preda-predatore
  Equazioni:

  d p / dt = a * p - c * p * P
  d P / dt = - b * P + e * p * P

 */

#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

#include "RK2.h"
#include "Integrali.h"


// Variabli globali (parametri dell'equazione, da utilizzare in dydt!)

double a, c, b, e, eps;


// Prototipo per funzione calc_max

void calc_max( double, double *, long, long&, long& );


/*

  Programma principale

*/

int main()
{

  cout << "Programma che integra le equazioni di un sistema preda-predatore" << endl;
  cout << "con uno schema di Runge-Kutta al secondo ordine." << endl;

  int Neq = 2;
  cout << "Inserire dt, Tend, Tprint: ";
  double dt, Tend, Tprint;
  cin >> dt >> Tend >> Tprint;

  long Nend   = long( Tend / dt );
  long Nprint = long( Tprint / dt );

  double epsf;
  cout << "Inserire il valore di a, c, b, e ed eps: ";
  cin >> a >> c >> b >> e >> epsf;

  cout << "Inserire le condizioni iniziali p, P: ";
  double y[Neq];
  cin >> y[0] >> y[1];

  RK2 soluzione = RK2( Neq, y );

  cout << "Nome del file di output: ";
  char nfile[256];
  cin >> nfile;

  double tc = 0.0, tn;
  eps = 0.0;

  // Apertura del file di output e scrittura condizioni iniziali

  ofstream fout;
  fout.open( nfile );
  fout.precision( 15 );
  //  fout << "#   t     p    P" << endl;
  fout << tc;
  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
  fout << endl;

  // Ciclo temporale

  for ( long n = 1; n <= Nend; n++ )
    {

      tc = ( n - 1 ) * dt;

      if ( tc >= Tend*0.5 ) eps = epsf;

      // Avanza di un passo lo schema

      soluzione.stepRK2( tc, dt );
      tn = n * dt;

      // Stampa ogni Tprint intervalli di stampa

      if ( ( n % Nprint ) == 0 )
	{
	  soluzione.getValue( y );
	  fout << tn;
	  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
	  fout << endl;
	}

    }

  fout.close();

  // Legge i dati memorizzati su file per calcolare i periodi

  long Nout = long( Tend / Tprint ) + 1;
  double *tt = new double[ Nout ];
  double *pp = new double[ Nout ];
  double *PP = new double[ Nout ];
  ifstream finp;
  finp.open( nfile );
  char dummy[256];
  //  finp >> dummy;
  //  cout << dummy << endl;
  for ( long i = 0; i < Nout; i++ )
    {
      finp >> tt[i] >> pp[i] >> PP[i];
    }
  finp.close();

  // Calcola la posizione dei massimi per calcolare il periodo

  long p1, p2, P1, P2;

  calc_max( Tprint, pp, Nout, p1, p2 );
  cout << "Posizione dei massimi per le prede: " << p1 << ", " << p2 << endl;
  double Tper = tt[p2] - tt[p1];
  cout << "Periodo per le prede: " << Tper << endl;
  long Npp = p2 - p1 + 1;
  double *fint = new double[Npp];
  for ( long i = p1; i <= p2; i++ ) fint[i-p1] = pp[i];
  cout << "<p> / T: " << Trapezi( fint, Tprint, Npp ) / Tper << endl;
  cout << "b / e: " << b / e << endl << endl;
  delete [] fint;

  calc_max( Tprint, PP, Nout, P1, P2 );
  cout << "Posizione dei massimi per i predatori: " << P1 << ", " << P2 << endl;
  Tper = tt[P2] - tt[P1];
  cout << "Periodo per i predatori: " << Tper << endl;
  Npp = P2 - P1 + 1;
  fint = new double[Npp];
  for ( long i = P1; i <= P2; i++ ) fint[i-P1] = PP[i];
  cout << "<P> / T: " << Trapezi( fint, Tprint, Npp-1 ) / Tper << endl;
  cout << "Rapporto a / c: " << a / c << endl;
  delete [] fint;

}


// Funzione dydt: calcola il secondo membro dell'equazione

void dydt( int Neq, double t, double * y, double * dy )
{
  dy[0] = a * y[0] - c * y[0] * y[1] - eps * y[0];
  dy[1] = - b * y[1] + e * y[0] * y[1] - eps * y[1];
}


// Funzione calc_max: calcola la posizione di due massimi consecutivi in un vettore

void calc_max( double dt, double *v, long N, long& m1, long& m2 )
{
  double der1, der2, der1p;

  der1p = ( v[2] - v[0] ) / ( 2.0 * dt );
  long i = 2;
  int max_found = 0;
  while ( ( i < N - 1 ) && ( max_found < 2 ) )
    {
      der1 = ( v[i+1] - v[i-1] ) / ( 2.0 * dt );
      der2 = ( v[i+1] + v[i-1] - 2.0 * v[i] ) / ( dt * dt );
      if ( ( der1 * der1p <= 0.0 ) && ( der2 < 0.0 ) )
	{
	  max_found++;
	  if ( max_found == 1 ) m1 = i;
	  if ( max_found == 2 ) m2 = i;
	}
      der1p = der1;
      i++;
    }
}
